/* eslint-disable-next-line @typescript-eslint/naming-convention */
const { S3Client, GetObjectCommand, ListObjectsV2Command, PutObjectCommand } = require('@aws-sdk/client-s3');

const fs = require('fs');
const path = require('path');
const CONFIG = require('./config');

const S3_PAGE_SIZE = 1000;

async function* getObjectKeys(client, bucket, prefix, startAfter = undefined) {
  const response = await client.send(new ListObjectsV2Command({
    /* eslint-disable @typescript-eslint/naming-convention */
    Bucket: bucket,
    Prefix: `${ prefix }/`,
    MaxKeys: S3_PAGE_SIZE,
    StartAfter: startAfter,
    /* eslint-enable @typescript-eslint/naming-convention */
  }));

  if (response.IsTruncated) {
    yield* getObjectKeys(client, bucket, prefix, response.Contents[S3_PAGE_SIZE - 1].Key);
  }
  else if (response.Contents) {
    yield* response.Contents.map(o => o.Key);
  }
}

class S3ClientWrapper {
  constructor(options) {
    this.options = options;
    this.client = new S3Client({ region: options.s3BucketRegion });
  }

  async pullReferenceSet() {
    for await (const key of getObjectKeys(this.client, this.options.s3BucketName, this.options.storageKey)) {
      console.log(`Downloading file "${ key }"`);
      const command = new GetObjectCommand({
        /* eslint-disable @typescript-eslint/naming-convention */
        Bucket: this.options.s3BucketName,
        Key: key,
        /* eslint-enable @typescript-eslint/naming-convention */
      });

      const response = await this.client.send(command);

      if (response.ContentType.match(/^application\/x-directory/i)) {
        // We'll create the necessary directory structure later.
        continue;
      }

      const filePath = path.join(CONFIG.DATA_BASE_DIR, key.replace(`${ this.options.storageKey }/`, ''));
      const dir = path.dirname(filePath);

      try {
        await fs.promises.stat(dir);
      }
      catch {
        await fs.promises.mkdir(dir, { recursive: true });
      }
      finally {
        response.Body.pipe(fs.createWriteStream(filePath));
      }
    }
  }

  async pushReferenceSet() {
    await this.pushDirectory(CONFIG.DATA_REFERENCE_DIR);
  }

  async pushDirectory(basePath) {
    for (const file of await fs.promises.readdir(basePath)) {
      const folderKey = path.basename(basePath);
      const sourcePath = path.join(basePath, file);
      const destPath = path.join(this.options.storageKey, folderKey, file);

      console.log(`Uploading file ${ sourcePath } to s3://${ this.options.s3BucketName }/${ destPath }`);

      const command = new PutObjectCommand({
        /* eslint-disable @typescript-eslint/naming-convention */
        Bucket: this.options.s3BucketName,
        Key: destPath,
        Body: await fs.promises.readFile(sourcePath),
        /* eslint-enable @typescript-eslint/naming-convention */
      });

      await this.client.send(command);
    }
  }
}

module.exports = {
  /* eslint-disable-next-line @typescript-eslint/naming-convention */
  S3ClientWrapper,
};
